﻿using System.Data.OleDb;

using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraEditors.Controls;
using DevExpress.XtraGrid.Columns;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 연결
        /// </summary>
        private OleDbConnection connection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.connection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=nwind.mdb");

            this.gridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                true                            // Editable
            );

            this.gridView.OptionsBehavior.AutoPopulateColumns = true;

            this.gridControl.DataSource = Product.GetProductList(this.connection);

            this.gridView.BestFitColumns();

            this.gridView.ValidateRow         += gridView_ValidateRow;
            this.gridView.InvalidRowException += gridView_InvalidRowException;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 그리드 뷰 행 검증하기 - gridView_ValidateRow(sender, e)

        /// <summary>
        /// 그리드 뷰 행 검증하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_ValidateRow(object sender, ValidateRowEventArgs e)
        {
            GridView gridView = sender as GridView;

            GridColumn unitsOnOrderColumn = gridView.Columns["UnitsOnOrder"];
            GridColumn unitsInStockColumn = gridView.Columns["UnitsInStock"];

            decimal unitsOnOrder = (decimal)gridView.GetRowCellValue(e.RowHandle, unitsOnOrderColumn);
            decimal unitsInStock = (decimal)gridView.GetRowCellValue(e.RowHandle, unitsInStockColumn);

            if(unitsOnOrder > unitsInStock)
            {
                e.Valid = false;

                gridView.SetColumnError(unitsInStockColumn, "주문 단위보다 큽니다."  );
                gridView.SetColumnError(unitsOnOrderColumn, "재고 단위보다 작습니다.");
            }
        }

        #endregion
        #region 그리드 뷰 무효 행 예외 처리하기 - gridView_InvalidRowException(sender, e)

        /// <summary>
        /// 그리드 뷰 무효 행 예외 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_InvalidRowException(object sender, InvalidRowExceptionEventArgs e)
        {
            e.ExceptionMode = ExceptionMode.DisplayError;
        }

        #endregion
    }
}